/****************************************************************************
** File:    tz_utl1.c
**
** Description: This module contains a function for getting to current
**              timezone string from the operating system.
**
**      Author: Fred Polizo, Jr.    FP Systems
****************************************************************************/

#include <stdio.h>
#include <time.h>

#include <oci.h>

/* Tested with Solaris 2.5.1 and Oracle 8.0.3 */

/****************************************************************************
** Function:    get_timezone
**
** Description: A function that fills a caller provided output buffer with
**              the current timezone.  It returns the alternate
**              'daylight savings' timezone, if that can be determined.
**
**              Upon return, a null-terminated string will be stored into the
**              caller provided buffer.
**              If there is an error getting the timezone string or the
**              maximum size of the buffer is too small for the timezone value,
**              the indicator variable will be set to OCI_IND_NULL,
**              indicating a NULL.
****************************************************************************/
void 
get_timezone (char *pTimezone,
              sb4 *pMaxLenTimezone, /* not including NULL terminator */
              sb2 *pIndTimezone)
{
    if ( *pMaxLenTimezone > 0 )
    {
        time_t      tmpTime;
        struct tm   *pTimeInfo;
        char        timezoneBuff[BUFSIZ];
        size_t      lenCopied;

        /* Get the current time and extract the timezone string from it */
        tmpTime = time (0);
        pTimeInfo = localtime (&tmpTime);
        lenCopied = strftime (timezoneBuff,
                              sizeof(timezoneBuff),
                              "%Z",
                              pTimeInfo);

        /* If the timezone string fits, copy it into the output parameter */
        if ( (lenCopied > 0) && (lenCopied <= (size_t)*pMaxLenTimezone) )
        {
            /* Copy the string and return the length */
            memcpy (pTimezone, timezoneBuff, lenCopied);

            /* Null-terminate the returned string. Note that the
            ** null-terminator byte is not included in MAXLEN */
            pTimezone[lenCopied] = '\0';

            *pIndTimezone = OCI_IND_NOTNULL;
        }
        else
        {
            /* it doesn't fit, so return a NULL */
            *pIndTimezone = OCI_IND_NULL;
        } 
    }
    else
    {
        /* the output parameter is too small, return a NULL */
        *pIndTimezone = OCI_IND_NULL;
    }
}